import Foundation
import Glibc

struct Commit {
	var name: String
	var email: String
	var message: String
	var date: String
}

struct GitCommits {
	var commits = [Commit]()

	init(repo: String) {
		commits.removeAll(keepCapacity: true)

		print("Fetching \(repo)…")

		let urlString = "https://api.github.com/repos/\(repo)/commits"
		system("curl -s \(urlString) > commits")

		let contents: String

		do {
			contents = try String(contentsOfFile: "commits", encoding: NSUTF8StringEncoding)
		} catch {
			print("Failed to load commits file!")
			exit(0)
		}

		if let json = JSON.parse(contents).0 {
			for (_, value) in json {
				let name = value["commit"]["committer"]["name"].string ?? "Anonymous"
				let email = value["commit"]["committer"]["email"].string ?? "tcook@apple.com"
				let date = value["commit"]["committer"]["date"].string ?? "1984-01-24T08:00:00Z"

				var message = value["commit"]["message"].string ?? "1984-01-24T08:00:00Z"
				message = message.stringByReplacingOccurrencesOfString("\n", withString: " ")

				let commit = Commit(name: name, email: email, message: message, date: date)
				commits.append(commit)
			}
		}


		// This is a closure that sorts the commits array by the name of the committer.
		// This is very similar to how the filtering takes place below, but here the code
		// is inline.
		commits.sortInPlace { (first, second) -> Bool in
			return first.name.caseInsensitiveCompare(second.name) == .OrderedAscending
		}
	}

	func present(filter: ((Commit) -> Bool)?) {
		let filteredCommits: [Commit]

		if let commitFilter = filter {
			// we were passed a filter, so use it!
			filteredCommits = commits.filter(commitFilter)
		} else {
			// no filter; just use the basic commits list
			filteredCommits = commits
		}

		for (index, commit) in filteredCommits.enumerate() {
			print("\(index + 1) \(commit.name) <\(commit.email)> at \(commit.date):")
			print("\t\(commit.message)\n")
		}

		showOptions()
	}

	func showOptions() {
		print("OPTIONS: '1' to show everything, '2' to show only Apple engineers, '3' to show only pull request merges, 'x' to quit.")
	}

	func mainLoop() {
		present(nil)

		mainLoop: while true {
			if let input = readLine() {
				switch input.lowercaseString {
				case "1":
					// passing nil here will show all commits
					present(nil)

				case "2":
					// look for "@apple" anywhere in the email address
					present { $0.email.containsString("@apple") }

				case "3":
					// look for messages that start with "Merge pull request"
					present { $0.message.hasPrefix("Merge pull request") }

				case "x":
					break mainLoop

				default:
					showOptions()
				}
			}
		}
	}
}

print("Please enter a GitHub repository to query:")

if let entry = readLine() {
	let commits = GitCommits(repo: entry)
	commits.mainLoop()
}
